﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우
    /// </summary>
    public partial class MainWindow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 스토리보트
        /// </summary>
        private Storyboard storyboard;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindow()
        {
            NameScope.SetNameScope(this, new NameScope());

            #region 버튼을 생성한다.

            Rectangle rectangle = new Rectangle();

            rectangle.Name   = "rectangle";
            rectangle.Width  = 100;
            rectangle.Height = 100;
            rectangle.Fill   = Brushes.Blue;

            #endregion

            RegisterName(rectangle.Name, rectangle);

            #region Begin 버튼을 생성한다.

            Button beginButton = new Button();

            beginButton.Width   = 100;
            beginButton.Height  = 30;
            beginButton.Content = "Begin";

            beginButton.Click += beginButton_Clicked;

            #endregion
            #region Pause 버튼을 생성한다.

            Button pauseButton = new Button();

            pauseButton.Margin  = new Thickness(10, 0, 0, 0);
            pauseButton.Width   = 100;
            pauseButton.Height  = 30;
            pauseButton.Content = "Pause";

            pauseButton.Click += pauseButton_Clicked;

            #endregion
            #region Resume 버튼을 생성한다.

            Button resumeButton = new Button();

            resumeButton.Margin  = new Thickness(10, 0, 0, 0);
            resumeButton.Width   = 100;
            resumeButton.Height  = 30;
            resumeButton.Content = "Resume";

            resumeButton.Click += resumeButton_Clicked;

            #endregion
            #region Skip to Fill 버튼을 생성한다.

            Button skipToFillButton = new Button();

            skipToFillButton.Margin  = new Thickness(10, 0, 0, 0);
            skipToFillButton.Width   = 120;
            skipToFillButton.Height  = 30;
            skipToFillButton.Content = "Skip to Fill";

            skipToFillButton.Click += new RoutedEventHandler(skipToFillButton_Clicked);

            #endregion
            #region Stop 버튼을 생성한다.

            Button stopButton = new Button();

            stopButton.Content = "Stop";

            stopButton.Margin  = new Thickness(10, 0, 0, 0);
            stopButton.Width   = 100;
            stopButton.Height  = 30;

            stopButton.Click += stopButton_Clicked;

            #endregion
            #region 컨트롤 스택 패널을 생성한다.

            StackPanel controlStackPanel = new StackPanel();

            controlStackPanel.Margin      = new Thickness(0, 50, 0, 0);
            controlStackPanel.Orientation = Orientation.Horizontal;

            controlStackPanel.Children.Add(beginButton);
            controlStackPanel.Children.Add(pauseButton);
            controlStackPanel.Children.Add(resumeButton);
            controlStackPanel.Children.Add(skipToFillButton);
            controlStackPanel.Children.Add(stopButton);

            #endregion
            #region 스택 패널을 생성한다.

            StackPanel stackPanel = new StackPanel();

            stackPanel.HorizontalAlignment = HorizontalAlignment.Center;
            stackPanel.VerticalAlignment   = VerticalAlignment.Center;

            stackPanel.Children.Add(rectangle);
            stackPanel.Children.Add(controlStackPanel);

            #endregion
            #region 더블 애니메이션을 생성한다.

            DoubleAnimation doubleAnimation = new DoubleAnimation();

            doubleAnimation.Duration = new Duration(TimeSpan.FromMilliseconds(5000));
            doubleAnimation.From     = 1.0;
            doubleAnimation.To       = 0.0;

            #endregion
            #region 스토리보드를 생성한다.

            storyboard = new Storyboard();

            storyboard.Children.Add(doubleAnimation);

            Storyboard.SetTargetName(doubleAnimation, rectangle.Name);
            Storyboard.SetTargetProperty(doubleAnimation, new PropertyPath(Rectangle.OpacityProperty));

            #endregion

            Content = stackPanel;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Begin 버튼 클릭시 처리하기 - beginButton_Clicked(sender, e)

        /// <summary>
        /// Begin 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void beginButton_Clicked(object sender, RoutedEventArgs e)
        {
            this.storyboard.Begin(this, true);
        }

        #endregion
        #region Pause 버튼 클릭시 처리하기 - pauseButton_Clicked(sender, e)

        /// <summary>
        /// Pause 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void pauseButton_Clicked(object sender, RoutedEventArgs e)
        {
            this.storyboard.Pause(this);
        }

        #endregion
        #region Resume 버튼 클릭시 처리하기 - resumeButton_Clicked(sender, e)

        /// <summary>
        /// Resume 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void resumeButton_Clicked(object sender, RoutedEventArgs e)
        {
            this.storyboard.Resume(this);
        }

        #endregion
        #region Skip To Fill 버튼 클릭시 처리하기 - skipToFillButton_Clicked(sender, e)

        /// <summary>
        /// Skip To Fill 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void skipToFillButton_Clicked(object sender, RoutedEventArgs e)
        {
            this.storyboard.SkipToFill(this);
        }

        #endregion
        #region Stop 버튼 클릭시 처리하기 - stopButton_Clicked(sender, e)

        /// <summary>
        /// Stop 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void stopButton_Clicked(object sender, RoutedEventArgs e)
        {
            this.storyboard.Stop(this);
        }

        #endregion
    }
}